@@ -1,9 +1,10 @@
 #![doc(html_playground_url = "https://play.rust-lang.org/")]
+/// 保持兼容性: use gostd::builtin 等同于 use gostd_builtin
+pub use gostd_builtin as builtin;
+/// 保存兼容性: use gostd::time 等同于 use gostd_time;
+pub use gostd_time as time;
 pub mod archive;
 pub mod bufio;
-#[macro_use]
-pub mod builtin;
-extern crate lazy_static;
 pub mod bytes;
 pub mod cmd;
 pub mod compress;
@@ -44,7 +45,6 @@ pub mod syscall;
 pub mod testdata;
 pub mod testing;
 pub mod text;
-pub mod time;
 pub mod unicode;
 #[cfg(test)]
 mod tests {
