@@ -203,7 +203,36 @@ use crate::strings;
 use crate::time;
 use std::collections::HashMap;
 use std::io::Error;
-
+/// Get issues a GET to the specified URL. If the response is one of the following redirect codes, Get follows the redirect,up to a maximum of 10 redirects:
+/// ```text
+/// 301 (Moved Permanently)
+/// 302 (Found)
+/// 303 (See Other)
+/// 307 (Temporary Redirect)
+/// 308 (Permanent Redirect)
+/// ```
+/// Get is a wrapper around Client.Get.
+/// <details class="rustdoc-toggle top-doc">
+/// <summary class="docblock">zh-cn</summary>
+/// Get向指定的URL发出一个GET请求，如果回应的状态码如下，Get会在调用c.CheckRedirect后执行重定向
+///Get是对包变量Client的Get方法的包装。
+/// </details>
+///
+/// # Example
+///
+/// ```
+/// use gostd::net::http;
+///
+/// fn main() -> Result<(), std::io::Error> {
+///     let url = "https://petstore.swagger.io/v2/pet/findByStatus?status=available";
+///     let response = http::Get(url)?;///
+///     println!(
+///         "{}",
+///         String::from_utf8(response.Body.expect("return body error")).unwrap()
+///     );
+///     Ok(())
+/// }
+/// ```
 pub fn Get(url: &str) -> HttpResult {
     Client::New().Get(url)
 }
@@ -212,6 +241,33 @@ pub fn Head(url: &str) -> HttpResult {
     Client::New().Head(url)
 }
 
+/// Post issues a POST to the specified URL. Post is a wrapper around DefaultClient.Post.
+/// To set custom headers, use Request::New and Client.Do.
+/// <details class="rustdoc-toggle top-doc">
+/// <summary class="docblock">zh-cn</summary>
+/// Post向指定的URL发出一个POST请求。bodyType为POST数据的类型， body为POST数据，作为请求的主体
+/// </details>
+///
+/// # Example
+///
+/// ```
+/// use gostd::net::http;
+/// fn main() -> Result<(), std::io::Error> {
+///     let url = "https://petstore.swagger.io/v2/pet";
+///     let postbody = r#"{"id":0,"category":{"id":0,"name":"string"},"name":"doggie","photoUrls":["string"],"tags":[{"id":0,"name":"string"}],"status":"available"}"#
+///    .as_bytes()
+///    .to_vec();
+///    let response = http::Post(url, "application/json", Some(postbody))?;
+///
+///    println!(
+///        "{}",
+///        String::from_utf8(response.Body.expect("return body error")).unwrap()
+///    );
+///
+///    Ok(())
+/// }
+///
+/// ```
 pub fn Post(url: &str, contentType: &str, body: Option<Vec<u8>>) -> HttpResult {
     Client::New().Post(url, contentType, body)
 }
@@ -251,13 +307,13 @@ impl Client {
 
     pub fn Get(&mut self, url: &str) -> HttpResult {
         let mut req = Request::New(Method::Get, url, None)?;
-        self.Do(&req)
+        self.Do(&mut req)
     }
 
     pub fn Post(&mut self, url: &str, contentType: &str, body: Option<Vec<u8>>) -> HttpResult {
         let mut req = Request::New(Method::Post, url, body)?;
         req.Header.Set("Content-Type", contentType);
-        self.Do(&req)
+        self.Do(&mut req)
     }
 
     pub fn PostForm(&mut self, url: &str, data: url::Values) -> HttpResult {
@@ -270,38 +326,38 @@ impl Client {
 
     pub fn Head(&mut self, url: &str) -> HttpResult {
         let mut req = Request::New(Method::Head, url, None)?;
-        self.Do(&req)
+        self.Do(&mut req)
     }
 
     pub fn Patch(&mut self, url: &str, body: Option<Vec<u8>>) -> HttpResult {
         let mut req = Request::New(Method::Patch, url, body)?;
-        self.Do(&req)
+        self.Do(&mut req)
     }
 
     pub fn Put(&mut self, url: &str, body: Option<Vec<u8>>) -> HttpResult {
         let mut req = Request::New(Method::Put, url, body)?;
-        self.Do(&req)
+        self.Do(&mut req)
     }
 
     pub fn Delete(&mut self, url: &str) -> HttpResult {
         let mut req = Request::New(Method::Delete, url, None)?;
-        self.Do(&req)
+        self.Do(&mut req)
     }
 
-    pub fn Do(&mut self, req: &Request) -> HttpResult {
+    pub fn Do(&mut self, req: &mut Request) -> HttpResult {
         self.done(req)
     }
 
     fn send(
         &mut self,
-        req: &Request,
+        req: &mut Request,
         deadline: time::Time,
     ) -> Result<(Response, fn() -> bool), Error> {
         let (resp, didTimeout) = send(req, self.transport(), deadline)?;
         Ok((resp, didTimeout))
     }
 
-    fn done(&mut self, req: &Request) -> HttpResult {
+    fn done(&mut self, req: &mut Request) -> HttpResult {
         let deadline = self.deadline();
         let (resp, didTimeout) = self.send(req, deadline)?;
         Ok(resp)
@@ -320,15 +376,47 @@ impl Client {
 }
 
 fn send(
-    ireq: &Request,
+    ireq: &mut Request,
     mut rt: Box<dyn RoundTripper>,
     deadline: time::Time,
 ) -> Result<(Response, fn() -> bool), Error> {
-    let resp = rt.RoundTrip(ireq)?;
+    let mut resp = Response::default();
     fn didTimeout() -> bool {
         return false;
     };
-    Ok((resp, didTimeout)) //didTimeout待修改
+    loop {
+        let mut resp = rt.RoundTrip(ireq)?;
+        let mut loc = resp.Header.Get("Location");
+        let (redirectMethod, shouldRedirect, includeBody) =
+            redirectBehavior(ireq.Method.as_str(), &resp, ireq);
+        if !shouldRedirect {
+            return Ok((resp, didTimeout));
+        }
+        let mut u = ireq.URL.Parse(loc.as_str())?;
+        let urlRef = refererForURL(&ireq.URL, &u);
+        ireq.Method = redirectMethod.clone();
+        ireq.URL = u.clone();
+        ireq.Header.Set("Referer", urlRef.as_str());
+    }
+}
+
+fn redirectBehavior(reqMethod: &str, resp: &Response, ireq: &Request) -> (String, bool, bool) {
+    let mut shouldRedirect = false;
+    let mut includeBody = false;
+    match resp.StatusCode {
+        301 | 302 | 303 => return (Method::Get.String().to_string(), true, false),
+        307 | 308 => {
+            if resp.Header.Get("Location") == "" {
+                shouldRedirect = true;
+                includeBody = false;
+            }
+            if ireq.Body.is_none() && ireq.ContentLength != 0 {
+                shouldRedirect = false;
+            }
+        }
+        _ => (),
+    }
+    (reqMethod.to_string(), shouldRedirect, includeBody)
 }
 
 pub trait RoundTripper {
@@ -368,6 +456,7 @@ pub struct Request {
     Trailer: Header,
     RemoteAddr: String,
     RequestURI: String,
+    isTLS: bool,
     // TLS *tls.ConnectionState,
     // Cancel <-chan struct{}
     // ctx context.Context
@@ -395,11 +484,15 @@ impl Request {
             RequestURI: "".to_string(),
             Body: None,
             Host: u.Host.to_owned(),
+            isTLS: false,
         };
         if let Some(buf) = body {
             req.ContentLength = len!(buf) as i64;
             req.Body = Some(buf);
         }
+        if strings::HasPrefix(url, "https://") {
+            req.isTLS = true
+        }
         Ok(req)
     }
 
@@ -752,6 +845,8 @@ impl Transport {
         };
         let cm = self.connectMethodForRequest(treq)?;
         let (mut pconn, mut conn) = self.getConn(treq, cm)?;
+        // conn.set_write_timeout(Some(std::time::Duration::new(5, 0)));
+        // conn.set_read_timeout(Some(std::time::Duration::new(5, 0)));
 
         pconn.roundTrip(treq, conn)
     }
@@ -808,6 +903,7 @@ impl Transport {
         4 << 10
     }
 }
+
 fn canonicalAddr(url: &url::URL) -> String {
     let portMap: HashMap<String, String> = [
         ("http".to_string(), "80".to_string()),
@@ -817,8 +913,8 @@ fn canonicalAddr(url: &url::URL) -> String {
     .iter()
     .cloned()
     .collect();
-    let addr = url.Hostname().clone();
-    let mut port = url.Port().clone();
+    let addr = url.Hostname().to_string();
+    let mut port = url.Port().to_string();
     if port == "" {
         port = portMap.get(url.Scheme.as_str()).unwrap().to_string();
     }
@@ -879,9 +975,15 @@ struct persistConn {
     reused: bool,
 }
 
+use rustls::ClientConnection;
+use rustls::StreamOwned;
+use std::convert::TryFrom;
 use std::io::prelude::*;
 use std::io::BufReader;
 use std::net::Shutdown;
+use std::net::TcpStream;
+use std::rc::Rc;
+use std::sync::Arc;
 impl persistConn {
     fn roundTrip(&mut self, req: &mut transportRequest, mut conn: TcpConn) -> HttpResult {
         self.numExpectedResponses += 1;
@@ -901,18 +1003,46 @@ impl persistConn {
             req.extra = Some(hd.clone());
             req.Req.Header = hd;
         }
-        let r = req.Req.Write()?;
-        // println!("{}", string(r.clone().as_slice()));
-        conn.write(r.as_slice())?;
-        let mut reader = BufReader::new(&conn);
-        let resp = ReadResponse(reader, &req.Req)?;
 
-        Ok(resp)
+        let r = req.Req.Write()?;
+
+        if req.Req.isTLS {
+            let mut tlsConn = getTLSConn(req.Req.Host.as_str(), conn);
+            tlsConn.write(r.as_slice())?;
+            let mut reader = BufReader::new(tlsConn);
+            let resp = ReadResponse(reader, &req.Req)?;
+            Ok(resp)
+        } else {
+            conn.write(r.as_slice())?;
+            let mut reader = BufReader::new(conn);
+            let resp = ReadResponse(reader, &req.Req)?;
+            Ok(resp)
+        }
     }
 }
+
 use std::io::ErrorKind;
 
-pub fn ReadResponse<'a>(mut r: BufReader<&TcpConn>, req: &'a Request) -> HttpResult {
+fn getTLSConn(dnsName: &str, socket: TcpConn) -> StreamOwned<ClientConnection, TcpConn> {
+    let mut clientRootCert = rustls::RootCertStore::empty();
+    clientRootCert.add_server_trust_anchors(webpki_roots::TLS_SERVER_ROOTS.0.iter().map(|ta| {
+        rustls::OwnedTrustAnchor::from_subject_spki_name_constraints(
+            ta.subject,
+            ta.spki,
+            ta.name_constraints,
+        )
+    }));
+    let tlsconfig = rustls::ClientConfig::builder()
+        .with_safe_defaults()
+        .with_root_certificates(clientRootCert)
+        .with_no_client_auth();
+    let serverName = rustls::ServerName::try_from(dnsName.as_ref()).expect("url error");
+    let mut tlsClient = ClientConnection::new(Arc::new(tlsconfig), serverName).unwrap();
+    let mut tlsConn = StreamOwned::new(tlsClient, socket);
+    tlsConn
+}
+
+pub fn ReadResponse(mut r: impl BufRead, req: &Request) -> HttpResult {
     let mut resp = Response::default();
     resp.Request = req.clone();
     // parse status line。
